/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 * 
 * The contents of this file are subject to the Mozilla Public License Version 
 * 1.1 (the "License"); you may not use this file except in compliance with 
 * the License. You may obtain a copy of the License at 
 * http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 * 
 * The Original Code is mozilla.org code.
 * 
 * The Initial Developer of the Original Code is
 * Netscape Communications Corporation.
 * Portions created by the Initial Developer are Copyright (C) 1996-2003
 * the Initial Developer. All Rights Reserved.
 * 
 * Contributor(s):
 *	Mark Smith <MarkCSmithWork@aol.com>
 * 
 * Alternatively, the contents of this file may be used under the terms of
 * either of the GNU General Public License Version 2 or later (the "GPL"),
 * or the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 * 
 * ***** END LICENSE BLOCK ***** */

#include <stdio.h>
#include <stdlib.h>
#include <sys/types.h>
#include <sys/stat.h>

main( argc, argv )
    int		argc;
    char	*argv[];
{
    struct stat	stbuf;
    char	at[ 255 ];
    char	mt[ 255 ];
    char	ct[ 255 ];
    int		printed = 0, numeric_flg = 0, errflg = 0, c;
    extern char	*optarg;
    extern int	optind, opterr, optopt;

    while ((c = getopt( argc, argv, "n" )) != EOF ) {
	switch( c ) {
	case 'n':
	    numeric_flg = 1;
	    break;
	default:
	    ++errflg;
	}
    }

    if ( errflg || optind >= argc ) {
	(void) fprintf( stderr, "usage: %s [-n] files...\n", argv[ 0 ] );
	exit( 1 );
    }

    for ( ; optind < argc; ++optind ) {
	if ( stat( argv[optind], &stbuf ) != 0 ) {
	    (void) perror( argv[optind] );
	} else {
	    if ( !printed ) {
		(void) printf( "%20s %20s %20s %s\n",
		    "access", "modify", "change", "name" );
		++printed;
	    }
	    if ( numeric_flg ) {
		(void) printf( "%20d %20d %20d %s\n",
			stbuf.st_atim.tv_sec, stbuf.st_mtim.tv_sec,
			stbuf.st_ctim.tv_sec, argv[optind] );
	    } else {
		strcpy( at, ctime( &stbuf.st_atim.tv_sec ) + 4 );
		strcpy( mt, ctime( &stbuf.st_mtim.tv_sec ) + 4 );
		strcpy( ct, ctime( &stbuf.st_ctim.tv_sec ) + 4 );
		(void) printf( "%.20s %.20s %.20s %s\n", at, mt, ct,
			argv[optind]);
	    }
	}
    }
    exit( 0 );
}
